! function to calculate CMOD5n
! Copyright (C) 2008   Ad Stoffelen (KNMI), Hans Hersbach (ECMWF),
!                      Jos de Kloe (KNMI), Siebren de Haan (KNMI),
!                      Anton Verhoef (KNMI)
! License: GPL (see file COPYING)

FUNCTION calc_sigma0_cmod5_n(v,phi,theta) RESULT(CMOD5_N)

  !-----------------------------------------------------
  ! *CMOD5_N* - GEOPHYSICAL MODEL FUNCTION FOR THE ERS AND ASCAT C-BAND SCATTEROMETERS
  !
  !     A. STOFFELEN              MAY  1991 ECMWF  CMOD4
  !     A. STOFFELEN, S. DE HAAN  DEC  2001 KNMI   CMOD5 PROTOTYPE
  !     H. HERSBACH               JUNE 2002 ECMWF  COMPLETE REVISION
  !     J. de Kloe                JULI 2003 KNMI,  rewritten in fortan90
  !     A. Verhoef                JAN  2008 KNMI,  CMOD5 for neutral winds
  !
  !**   PURPOSE
  !     -------
  !     SIMULATION OF BACKSCATTER GIVEN A WIND VECTOR AND ICIDENCE ANGLE.
  !     CMOD5_N IS TO BE USED IN AN INVERSION ROUTINE IN ORDER TO OBTAIN
  !     WIND VECTORS FROM BACKSCATTER TRIPLETS.
  !
  !**   INTERFACE
  !     ---------
  !     CMOD5_N(v, phi, theta)
  !         inputs:
  !              v     in [m/s] wind velocity (always >= 0)
  !              phi   in [deg] angle between azimuth and wind direction
  !                    (= D - AZM)
  !              theta in [deg] incidence angle
  !         output:
  !              CMOD5_N NORMALIZED BACKSCATTER (LINEAR)
  !---------------------------------------------------------------------

  implicit none

  real*4, parameter :: DTOR   = 57.29577951
  real*4, parameter :: THETM  = 40.
  real*4, parameter :: THETHR = 25.
  real*4, parameter :: ZPOW   = 1.6

  ! Coefficients obtained from Hans Hersbach by Email on 5 Feb 2008
  real*4, parameter :: C(28) = &
          (/ -0.6878, -0.7957,  0.3380, -0.1728, 0.0000,  0.0040, 0.1103, &
              0.0159,  6.7329,  2.7713, -2.2885, 0.4971, -0.7250, 0.0450, &
              0.0066,  0.3222,  0.0120, 22.7000, 2.0813,  3.0000, 8.3659, &
             -3.3428,  1.3236,  6.2437,  2.3893, 0.3249,  4.1590, 1.6930/)

  real*4 :: CMOD5_N, v, phi, theta
  real*4 :: A0, A1, A2, A3, B0, B1, B2
  real*4 :: FI, CSFI, CS2FI, X, XX
  real*4 :: GAM, S0, S
  real*4 :: D1, D2, V0, V2

  real*4, save :: Y0 = C(19)
  real*4, save :: PN = C(20)
  real*4, save :: A  = C(19)-(C(19)-1.)/C(20)

! WARNING: in an initialisation constant expression, exponents are
!          required to be of integer type, so the next line is
!          illegal fortran90 (SGI allows this, but Linux gives an error)
!  real*4, save :: B  = 1./(C(20)*(C(19)-1.)**(C(20)-1.))
! so I replaced c(20) by the constant value of "3"
  real*4, save :: B  = 1./(C(20)*(C(19)-1.)**(3-1))

  !  ANGLES
  FI=phi/DTOR
  CSFI = COS(FI)
  CS2FI= 2.00 * CSFI * CSFI - 1.00

  X  = (theta - THETM) / THETHR
  XX = X*X

  ! B0: FUNCTION OF WIND SPEED AND INCIDENCE ANGLE

  A0 =C( 1)+C( 2)*X+C( 3)*XX+C( 4)*X*XX
  A1 =C( 5)+C( 6)*X
  A2 =C( 7)+C( 8)*X

  GAM=C( 9)+C(10)*X+C(11)*XX
  S0 =C(12)+C(13)*X

  S = A2*V
  A3=1./(1.+EXP(-MAX(S,S0)))
  IF (S.LT.S0) A3=A3*(S/S0)**( S0*(1.- A3))

  B0=(A3**GAM)*10.**(A0+A1*V)

  !  B1: FUNCTION OF WIND SPEED AND INCIDENCE ANGLE

  B1 = C(15)*V*(0.5+X-TANH(4.*(X+C(16)+C(17)*V)))
  B1 = C(14)*(1.+X)- B1
  B1 = B1/(EXP( 0.34*(V-C(18)) )+1.)

  !  B2: FUNCTION OF WIND SPEED AND INCIDENCE ANGLE

  V0 = C(21) + C(22)*X + C(23)*XX
  D1 = C(24) + C(25)*X + C(26)*XX
  D2 = C(27) + C(28)*X

  V2 = (V/V0+1.)
  IF   (V2.LT.Y0) V2 = A+B*(V2-1.)**PN

  B2 = (-D1+D2*V2)*EXP(-V2)

  !  CMOD5_N: COMBINE THE THREE FOURIER TERMS

  CMOD5_N = B0*(1.0+B1*CSFI+B2*CS2FI)**ZPOW

  RETURN
END FUNCTION calc_sigma0_cmod5_n
